SystemOrganization addCategory: #'CS528-UT3AI'!

Error subclass: #AIError
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CS528-UT3AI'!

!AIError commentStamp: '<historical>' prior: 0!
An AIError is error for when the AI encounters a problem.

Instance Variables
!

Object subclass: #UT3AI
	instanceVariableNames: 'board piece'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CS528-UT3AI'!

!UT3AI commentStamp: '<historical>' prior: 0!
An UT3AI is a package for the Ultimate tic-tac-toe chess AI decision making.

Instance Variables
	gameBoard:		<Object>

gameBoard
	- xxxxx
!

!UT3AI class methodsFor: 'as yet unclassified' stamp: 'ShineLow 7/8/2013 00:42'!
new
	AIError signal: 'Instantiate with newWith: aBoard as: aPiece'! !

!UT3AI class methodsFor: 'as yet unclassified' stamp: 'ShineLow 7/8/2013 01:33'!
newWith: aBoard as: aPiece
	^ self basicNew board: aBoard piece: aPiece! !

!UT3AI methodsFor: 'initialize-release' stamp: 'ShineLow 7/8/2013 00:44'!
board: aBoard piece: aPiece
	board := aBoard.
	piece := aPiece.! !

!UT3AI methodsFor: 'public' stamp: 'ShineLow 7/8/2013 00:41'!
move
	^ self subclassResponsibility ! !

UT3AI subclass: #UT3NestedRandomAI
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CS528-UT3AI'!

!UT3NestedRandomAI commentStamp: '<historical>' prior: 0!
An UT3NestedRandomAI is yet another dumb AI, but works on the nested board.

Instance Variables
!

!UT3NestedRandomAI methodsFor: 'public' stamp: 'ShineLow 7/8/2013 01:27'!
move
	|size available nextMove|
	size := board size.
	available := Set new.
	1 to: size do: [ :row |
		1 to: size do: [ :column |
			(board canMoveAt: { row. column })
			ifTrue: [ 
				1 to: size do: [ :subRow |
					1 to: size do: [ :subColumn |
						( board canMoveAt: { row. column } subBoard: { subRow. subColumn } )
						ifTrue: [ available add: { { row. column }. { subRow. subColumn } } ]
						 ]
					 ]
				 ]
			 ]
		 ].
	nextMove := available atRandom.
	board at: { nextMove first first. nextMove first second } subBoard: { nextMove second first. nextMove second second } put: piece! !

UT3AI subclass: #UT3RandomAI
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CS528-UT3AI'!

!UT3RandomAI commentStamp: '<historical>' prior: 0!
An UT3RandomAI is a dumb AI, that just makes a random move on any moveable space.

Instance Variables
!

!UT3RandomAI methodsFor: 'public' stamp: 'ShineLow 7/8/2013 00:54'!
move 
	"Dumb logic here. Just find coordinates that can be moved to, then move there."
	|available nextMove|
	available := Set new.
	1 to: board size do: [ :aRow |
		1 to: board size do: [ :aColumn |
			(board canMoveAtRow: aRow column: aColumn)
			ifTrue: [ available add: { aRow. aColumn } ]
			 ]
		 ].
	available size = 0 ifTrue: [ AIError signal: 'Board is full!!' ].
	nextMove := available atRandom.
	board atRow: nextMove first column: nextMove second put: piece.! !

TestCase subclass: #UT3AITest
	instanceVariableNames: 'gameBoard'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CS528-UT3AI'!

!UT3AITest commentStamp: '<historical>' prior: 0!
An UT3AITest is the test package for UT3AI which is a package for the Ultimate tic-tac-toe chess AI decision making

Instance Variables
	gameBoard:		<Object>

gameBoard
	- xxxxx
!

!UT3AITest methodsFor: 'tests' stamp: 'Oliver 7/5/2013 19:57'!
test1
	| board1 system1 |
	system1 := UT3System new.
	board1 := system1 board. 
	UT3AI new initialize: board1.! !

UT3AITest subclass: #UT3NestedRandomAITest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CS528-UT3AI'!

!UT3NestedRandomAITest methodsFor: 'as yet unclassified' stamp: 'ShineLow 7/8/2013 01:37'!
test2AIsFillUpBoard
	"Same test as the normal RandomAI, just keep playing till entire board is full."
	|board ai1 ai2 currentMove|
	board := UT3NestedBoard new.
	ai1 := UT3NestedRandomAI newWith: board as: #cross.
	ai2 := UT3NestedRandomAI newWith: board as: #circle.
	currentMove := 0.
	[ board isFull ] whileFalse: [ 
		(currentMove % 2 = 0)
		ifTrue: [ ai1 move ]
		ifFalse: [ ai2 move ].
		currentMove := currentMove + 1
		 ].
	Transcript show: board printBoardString.
	! !

UT3AITest subclass: #UT3RandomAITest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CS528-UT3AI'!

!UT3RandomAITest methodsFor: 'as yet unclassified' stamp: 'ShineLow 7/8/2013 00:56'!
test2AIsFillUpBoard
	|board ai1 ai2 totalMoves crossCount circleCount |
	board := UT3BasicBoard new.
	ai1 := UT3RandomAI newWith: board as: #cross.
	ai2 := UT3RandomAI newWith: board as: #circle.
	
	"Depending on board size, we might not have equal number of moves"
	totalMoves := ( board size * board size ).
	(totalMoves % 2 = 0)
	ifTrue: [ 
		1 to: ( totalMoves / 2 ) do: [ :unused_move | ai1 move. ai2 move. ] ]
	ifFalse: [ 
		ai1 move.
		1 to: ( totalMoves - 1 / 2 ) do: [ :unused_move | ai2 move. ai1 move. ] ].
	
	"When done, the board should be full."
	self assert: board isFull.
	Transcript show: board printBoardString.
	crossCount := 0.
	circleCount := 0.
	
	board indicesDo: [ :aRow :aCol |
		( (board atRow: aRow column: aCol) state = #cross )
		ifTrue: [ crossCount := crossCount + 1 ]
		ifFalse: [ circleCount := circleCount + 1 ] ].
	self assert: ( crossCount - circleCount <= 1 )
! !

!UT3RandomAITest methodsFor: 'as yet unclassified' stamp: 'ShineLow 7/8/2013 00:16'!
testAIFillsUpBoard
	|board ai|
	board := UT3BasicBoard new.
	ai := UT3RandomAI newWith: board as: #cross.
	1 to: ( board size * board size ) do: [ 
		:unused_each | ai move ].
	self assert: board isFull.! !

!UT3RandomAITest methodsFor: 'as yet unclassified' stamp: 'ShineLow 7/8/2013 00:39'!
testBasicWhenFullThrowsError
	|board ai|
	board := UT3BasicBoard new.
	ai := UT3RandomAI newWith: board as: #cross.
	1 to: ( board size * board size ) do: [ 
		:unused_each | ai move ].
	self assert: board isFull.
	self should: [ ai move ] raise: AIError! !
